package no.tornado.query.typehandler;

import java.util.HashMap;
import java.util.Map;

public class TypeMap {
    private static Map<Class, Map<Class, TypeHandler>> javaHandlers = new HashMap<>();
    private static Map<Class, Map<Class, TypeHandler>> sqlHandlers = new HashMap<>();
    
    public static void register(TypeHandler typeHandler) {
        Map<Class, TypeHandler> handlersForJavaType = javaHandlers.get(typeHandler.javaClass());
        if (handlersForJavaType == null) {
            handlersForJavaType = new HashMap<>();
            javaHandlers.put(typeHandler.javaClass(), handlersForJavaType);
        }
        handlersForJavaType.put(typeHandler.sqlClass(), typeHandler);
        
        Map<Class, TypeHandler> handlersForSqlType = sqlHandlers.get(typeHandler.sqlClass());
        if (handlersForSqlType == null) {
            handlersForSqlType = new HashMap<>();
            sqlHandlers.put(typeHandler.sqlClass(), handlersForSqlType);
        }
        handlersForSqlType.put(typeHandler.javaClass(), typeHandler);
    }
    
    public static TypeHandler getHandler(Class javaClass, Class sqlClass) {
        if (javaClass == null)
            return null;

        if (sqlClass == null)
            return getFirst(javaHandlers.get(javaClass));

        Map<Class, TypeHandler> sqlHandlersForJavaType = javaHandlers.get(javaClass);
        if (sqlHandlersForJavaType == null || sqlHandlersForJavaType.isEmpty())
            return null;

        return sqlHandlersForJavaType.get(sqlClass);
    }

    private static TypeHandler getFirst(Map<Class, TypeHandler> map) {
        if (map == null || map.isEmpty())
            return null;
        else
            return map.values().iterator().next();
    }

    static {
        register(new DateToSqlDateTypeHandler());
        register(new DateToTimestampHandler());
        register(new IntegerTypeHandler());
        register(new BooleanTypeHandler());
        register(new LongToIntegerTypeHandler());
    }
}
